@@ -2312,6 +2312,30 @@ gsm_manager_shutdown (GsmManager *manager,
         return TRUE;
 }
 
+gboolean
+gsm_manager_can_shutdown (GsmManager *manager,
+                          gboolean   *shutdown_available,
+                          GError    **error)
+{
+        GsmConsolekit *consolekit;
+        GsmPowerManager *power_manager;
+
+        g_debug ("GsmManager: CanShutdown called");
+
+        g_return_val_if_fail (GSM_IS_MANAGER (manager), FALSE);
+
+        consolekit = gsm_get_consolekit ();
+        power_manager = gsm_get_power_manager ();
+        *shutdown_available = gsm_consolekit_can_stop (consolekit)
+                              || gsm_consolekit_can_restart (consolekit)
+                              || gsm_power_manager_can_suspend (power_manager)
+                              || gsm_power_manager_can_hibernate (power_manager);
+        g_object_unref (consolekit);
+        g_object_unref (power_manager);
+
+        return TRUE;
+}
+
 gboolean
 gsm_manager_logout (GsmManager *manager,
                     guint       logout_mode,
@@ -2412,13 +2436,10 @@ gsm_manager_register_client (GsmManager            *manager,
 
         g_debug ("GsmManager: Adding new client %s to session", new_startup_id);
 
-        if (IS_STRING_EMPTY (startup_id) && IS_STRING_EMPTY (app_id)) {
-                /* just accept the client - we can't associate with an
-                   existing App */
-                app = NULL;
-        } else if (IS_STRING_EMPTY (startup_id)) {
+        if (app == NULL && !IS_STRING_EMPTY (startup_id)) {
                 app = find_app_for_startup_id (manager, startup_id);
-        } else if (IS_STRING_EMPTY (startup_id)) {
+        }
+        if (app == NULL && !IS_STRING_EMPTY (app_id)) {
                 /* try to associate this app id with a known app */
                 app = find_app_for_app_id (manager, app_id);
         }
